library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity divide is
  port(Clock: in std_logic;
       s: in std_logic;
       DataA: in std_logic_vector(7 downto 0);
       DataB: in std_logic_vector(7 downto 0);
       R: out std_logic_vector(7 downto 0);
       Q:out std_logic_vector(7 downto 0));
end divide;

architecture divide_arch of divide is
  component Aregister
    port(Clock: in std_logic;
         EA: in std_logic;
         LA: in std_logic;
         DataA: in std_logic_vector(7 downto 0);
         A: out std_logic_vector(7 downto 0));
  end component;
  
  component Bregister
    port(Clock: in std_logic;
         EB: in std_logic;
         DataB: in std_logic_vector(7 downto 0);
         B: out std_logic_vector(7 downto 0));
  end component;
  
  component Rregister
    port(Clock: in std_logic;
         ER: in std_logic;
         LR: in std_logic;
         w: in std_logic;
         DataR: in std_logic_vector(7 downto 0);
         R: out std_logic_vector(7 downto 0));
  end component;
  
  component Qregister
    port(Clock: in std_logic;
         EQ: in std_logic;
         w: in std_logic;
         Q: out std_logic_vector(7 downto 0));
  end component;
  
  component adder8b is
    port(X: in  STD_LOGIC_VECTOR(7 downto 0);
         Y: in  STD_LOGIC_VECTOR(7 downto 0);
         CIN: in  STD_LOGIC;
         S: out STD_LOGIC_VECTOR(7 downto 0);
         COUT: out STD_LOGIC);
  end component;
  
  component multiplexer is
    port(Rsel:in std_logic;
         X: in std_logic_vector(7 downto 0);
         Y: in std_logic_vector(7 downto 0);
         O: out std_logic_vector(7 downto 0));
  end component;
  
  component counter
    port(Clock: in std_logic;
         LC: in std_logic;
         EC: in std_logic;
         z: out std_logic);
  end component;
  
  component statemachine
    port(Clock: in std_logic;
         s: in std_logic;
         z: in std_logic;
         cout: in std_logic;
         Rsel: out std_logic;
         LR: out std_logic;
         LC: out std_logic;
         ER: out std_logic;
         EA: out std_logic;
         EQ: out std_logic;
         EC: out std_logic);
  end component;
  
  signal z: std_logic;
  signal cout: std_logic;
  signal Rsel: std_logic;
  signal LR: std_logic;
  signal LC: std_logic;
  signal ER: std_logic;
  signal EA: std_logic;
  signal EQ: std_logic;
  signal EC: std_logic;
  signal A: std_logic_vector(7 downto 0);
  signal B:std_logic_vector(7 downto 0);
  signal BN:std_logic_vector(7 downto 0);
  signal IR:std_logic_vector(7 downto 0);
  signal DataR: std_logic_vector(7 downto 0);
  signal Rmux: std_logic_vector(7 downto 0);
  
  begin
    U1: Aregister port map(Clock=>Clock,EA=>EA,LA=>LC,DataA=>DataA,A=>A);
    U2: Bregister port map(Clock=>Clock,EB=>LC,DataB=>DataB,B=>B);
    U3: Rregister port map(Clock=>Clock,ER=>ER,LR=>LR,w=>A(7),DataR=>DataR,R=>IR);
    U4: Qregister port map(Clock=>Clock,EQ=>EQ,w=>cout,Q=>Q);
    BN<=not B;
    U5: adder8b port map(X=>IR,Y=>BN,cin=>'1',S=>Rmux,cout=>cout);
    U6: multiplexer port map(Rsel=>Rsel,X=>"00000000",Y=>Rmux,O=>DataR);
    U7: counter port map(Clock=>Clock,LC=>LC,EC=>EC,z=>z);
    U8: statemachine port map(Clock=>Clock,s=>s,z=>z,cout=>cout,Rsel=>Rsel,
                              LR=>LR,LC=>LC,ER=>ER,EA=>EA,EQ=>EQ,EC=>EC);
    R<=IR;
end divide_arch;